package com.halo.halojuhesdk.core.base.utils;

import static com.halo.halojuhesdk.core.base.utils.FileUtils.getExternalStoragePath;

import android.content.Context;
import android.content.pm.PackageManager;
import android.text.TextUtils;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.Closeable;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.Writer;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.Properties;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;
import java.util.zip.InflaterInputStream;


/**
 * 项目名称：SearchDemo 类名：HttpUtils 创建人：Liuzheng 创建时间：2015-1-10 下午1:13:02
 */
public class Utils {

    private static final String TAG = "Utils";

    // 保存用户设置的SeID


    /**
     * @description 将String进行Gzip压缩
     * @version
     * @author Wang mingjun
     * @tags
     * @see
     * @since 2015-10-21
     */
    public static String gzipString(String content) {
        String resault = null;
        if (!TextUtils.isEmpty(content)) {
            ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
            GZIPOutputStream gzipOutputStream = null;
            try {
                gzipOutputStream = new GZIPOutputStream(outputStream);
                gzipOutputStream.write(content.getBytes());

            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                try {
                    if (gzipOutputStream != null) {
                        gzipOutputStream.close();
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            resault = new String(outputStream.toByteArray());
            if (outputStream != null) {
                try {
                    outputStream.close();
                } catch (IOException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            }
        }

        return resault;
    }

    public static int getResource(Context context, String defType, String name) {
        if ((context == null) || (defType == null)) {
            return -1;
        }
        return context.getResources().getIdentifier(name, defType,
                context.getPackageName());
    }

    protected static char hexDigits[] = {'0', '1', '2', '3', '4', '5', '6',
            '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};
    protected static MessageDigest messagedigest = null;

    static {
        try {
            messagedigest = MessageDigest.getInstance("MD5");
        } catch (NoSuchAlgorithmException e) {
            System.out.println("Ex");
        }
    }

    public static String getFileMD5String(File file) throws IOException {
        FileInputStream in = new FileInputStream(file);
        FileChannel ch = in.getChannel();
        MappedByteBuffer byteBuffer = ch.map(FileChannel.MapMode.READ_ONLY, 0,
                file.length());
        messagedigest.update(byteBuffer);
        String md5str = bufferToHex(messagedigest.digest());
        if (in != null) {
            in.close();
        }
        return md5str;
    }

    public static String getMD5String(String s) {
        return getMD5String(s.getBytes());
    }

    public static String getMD5String(byte[] bytes) {
        messagedigest.update(bytes);
        return bufferToHex(messagedigest.digest());
    }

    private static String bufferToHex(byte bytes[]) {
        return bufferToHex(bytes, 0, bytes.length);
    }

    private static String bufferToHex(byte bytes[], int m, int n) {
        StringBuffer stringbuffer = new StringBuffer(2 * n);
        int k = m + n;
        for (int l = m; l < k; l++) {
            appendHexPair(bytes[l], stringbuffer);
        }
        return stringbuffer.toString();
    }

    private static void appendHexPair(byte bt, StringBuffer stringbuffer) {
        char c0 = hexDigits[(bt & 0xf0) >> 4];
        char c1 = hexDigits[bt & 0xf];
        stringbuffer.append(c0);
        stringbuffer.append(c1);
    }

    public static boolean checkPassword(String password, String md5PwdStr) {
        String s = getMD5String(password);
        return s.equals(md5PwdStr);
    }

    /**
     * 获取 参数的UTF-8的EnCode值 如果失败返回“”
     *
     * @param content
     * @return
     */
    public static String encode(String content) {
        try {
            content = URLEncoder.encode(content, "UTF-8");
        } catch (Exception e) {
            e.printStackTrace();
            content = "";
        }
        return content;
    }

    /**
     * 获取 参数的UTF-8的DeCode值 如果失败返回“”
     *
     * @param content
     * @return
     */
    public static String decode(String content) {
        try {
            content = URLDecoder.decode(content, "UTF-8");
        } catch (Exception e) {
            content = "";
        }
        return content;
    }

    /**
     * @description 外层使用ByteArrayEntity包装压缩内容。如使用HttpEntity(即StringEntity)会导致服务器端解析压缩内容出错
     * (压缩格式不匹配) 。
     * @version
     * @author Liu zheng
     * @tags
     * @see 。采用byteArray，经测试与服务器端匹配。
     * 上面的压缩方法返回String对象（包装成String）。可与上面的压缩方法合并为统一格式返回。
     * @since 2015-11-5
     */
    public static byte[] gZip(byte[] data) {
        byte[] b = null;
        try {
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            GZIPOutputStream gzip = new GZIPOutputStream(bos);
            gzip.write(data);
            gzip.finish();
            gzip.close();
            b = bos.toByteArray();
            bos.close();
        } catch (Exception ex) {
            ex.printStackTrace();
            return b;
        }
        return b;
    }

    /**
     * 判断是否存在sd卡
     *
     * @return
     */
    public static boolean sdIsExist() {
        // 判断是否存在sd卡
        boolean sdExist = android.os.Environment.MEDIA_MOUNTED
                .equals(android.os.Environment.getExternalStorageState());
        if (!sdExist) {// 如果不存在,
            LogUtils.e( "SD卡不存在，请加载SD卡");
            return false;
        }
        return true;
    }

    /**
     * 使用文件通道的方式复制文件
     *
     * @param s 源文件
     * @param t 复制到的新文件
     */

    public static void fileChannelCopy(File s, File t) {

        FileInputStream fi = null;
        FileOutputStream fo = null;
        FileChannel in = null;
        FileChannel out = null;
        try {
            fi = new FileInputStream(s);
            fo = new FileOutputStream(t);
            in = fi.getChannel();// 得到对应的文件通道
            out = fo.getChannel();// 得到对应的文件通道
            in.transferTo(0, in.size(), out);// 连接两个通道，并且从in通道读取，然后写入out通道
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (fi != null)
                    fi.close();
                if (in != null)
                    in.close();
                if (fo != null)
                    fo.close();
                if (out != null)
                    out.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 把二进制byte数组生成 md5 32位 十六进制字符串，单个字节小于0xf，高位补0。
     *
     * @param bytes     输入
     * @param upperCase true：大写， false 小写字符串
     * @return 把二进制byte数组生成 md5 32位 十六进制字符串，单个字节小于0xf，高位补0。
     */
    public static String toMd5(byte[] bytes, boolean upperCase) {
        try {
            MessageDigest algorithm = MessageDigest.getInstance("MD5");
            algorithm.reset();
            algorithm.update(bytes);
            return toHexString(algorithm.digest(), "", upperCase);
        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 把二进制byte数组生成十六进制字符串，单个字节小于0xf，高位补0。
     *
     * @param bytes     输入
     * @param separator 分割线
     * @param upperCase true：大写， false 小写字符串
     * @return 把二进制byte数组生成十六进制字符串，单个字节小于0xf，高位补0。
     */
    private static String toHexString(byte[] bytes, String separator,
                                      boolean upperCase) {
        StringBuilder hexString = new StringBuilder();
        for (byte b : bytes) {
            String str = Integer.toHexString(0xFF & b); // SUPPRESS CHECKSTYLE
            if (upperCase) {
                str = str.toUpperCase();
            }
            if (str.length() == 1) {
                hexString.append("0");
            }
            hexString.append(str).append(separator);
        }
        return hexString.toString();
    }


    /**
     * 检查权限(重量级)。如果没有声明对应的权限，直接抛出 runtime exception。
     *
     * @param context    Context
     * @param permission android.Manifest.permission.xxxx
     */
    public static void checkPermission(Context context, String permission) {
        boolean allowedByPermission = isPermissionAllowed(context, permission);
        if (!allowedByPermission) {
            throw new SecurityException(
                    "Permission Denial: requires permission " + permission);
        }
    }

    /**
     * 检查权限(轻量级)。
     *
     * @param context    Context
     * @param permission android.Manifest.permission.xxxx
     * @return 是否具备权限
     */
    private static boolean isPermissionAllowed(Context context,
                                               String permission) {
        int result = context.checkCallingOrSelfPermission(permission);
        return result == PackageManager.PERMISSION_GRANTED;
    }

    public static Context getApplicationContext(Context context) {
        Context cont = context.getApplicationContext();
        if (cont == null) {
            return context;
        } else {
            return cont;
        }
    }


    public static BufferedInputStream decodeInputStream(InputStream inputStream, String contentEncoding) throws IOException {
        if (TextUtils.isEmpty(contentEncoding)) {
            return new BufferedInputStream(inputStream);
        } else if (contentEncoding.toLowerCase().equals("gzip")) {
            return new BufferedInputStream(new GZIPInputStream(inputStream));
        } else if (contentEncoding.toLowerCase().equals("deflate")) {
            return new BufferedInputStream(new InflaterInputStream(inputStream));
        } else {
            return new BufferedInputStream(inputStream);
        }
    }

    public static String parseInputString(BufferedInputStream inputStream) {
        String response = null;
        if (inputStream != null) {
            BufferedReader reader = null;
            try {

                StringBuilder sb = new StringBuilder();
                reader = new BufferedReader(new InputStreamReader(
                        inputStream, "UTF-8"));

                String line = null;
                while ((line = reader.readLine()) != null) {
                    LogUtils.i( line);
                    sb.append(line);
                }

                response = new String(sb);
            } catch (IllegalStateException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                closeStream(inputStream);
                closeStream(reader);
            }
        }
        return response;
    }

    public static void closeStream(Closeable closeable) {
        if (closeable != null) {
            try {
                closeable.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    public static Properties getProperties(Context cx, String S_Path) {
        InputStream is = null;
        try {
            if (cx == null) {
                return null;
            }
            FileUtils.createDirs(getExternalStoragePath());
            File file = new File(getExternalStoragePath() + File.separator + S_Path);
            if (!file.exists() && !FileUtils.copyFile(cx.getAssets().open(S_Path),file)) {
                return null;
            }
            is = new BufferedInputStream(new FileInputStream(file));
            Properties p = new Properties();
            p.load(is);
            return p;
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return null;
    }

    public static String readExceptionInfo(Throwable ex) {

        StringBuffer sb = new StringBuffer();
        Writer writer = new StringWriter();
        PrintWriter printWriter = new PrintWriter(writer);
        ex.printStackTrace(printWriter);
        Throwable cause = ex.getCause();
        while (cause != null) {
            cause.printStackTrace(printWriter);
            cause = cause.getCause();
        }
        printWriter.close();
        String result = writer.toString();
        sb.append(result);
        return sb.toString();
    }

    public static String formatCurrentTime() {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.CHINA);
        Date date = new Date(System.currentTimeMillis());
        return simpleDateFormat.format(date);
    }
}
