package com.halo.halojuhesdk.core.channel.applovin;

import android.app.Activity;
import android.util.Log;
import android.view.ViewGroup;
import android.widget.FrameLayout;

import androidx.annotation.NonNull;
import androidx.annotation.TransitionRes;

import com.applovin.mediation.MaxAd;
import com.applovin.mediation.MaxAdFormat;
import com.applovin.mediation.MaxAdRevenueListener;
import com.applovin.mediation.MaxAdViewAdListener;
import com.applovin.mediation.MaxError;
import com.applovin.mediation.ads.MaxAdView;
import com.applovin.sdk.AppLovinSdkUtils;
import com.halo.halojuhesdk.core.base.utils.CompareEcpmUtils;
import com.halo.halojuhesdk.core.base.utils.LogUtils;
import com.halo.halojuhesdk.core.channel.IBannerAd;
import com.halo.halojuhesdk.core.channel.IBannerProxyListener;
import com.halo.halojuhesdk.core.request.EcpmCompareBean;
import com.halo.halojuhesdk.core.request.InitDataProcessor;
import com.halo.halojuhesdk.core.request.SlotBean;
import com.halo.halojuhesdk.sdk.HlAdClient;

public class AppLovinBanner implements IBannerAd, MaxAdViewAdListener {
    private final Activity mActivity;
    private MaxAdView adBannerView;
    private SlotBean mLovinBean;
    private IBannerProxyListener mBannerListener;
    private boolean isShow = true;
    private boolean isClick = false;
    private MaxAd mMaxAd;
    private boolean isAdLoad;

    public AppLovinBanner(Activity activity, SlotBean bean, IBannerProxyListener listener) {
        mActivity = activity;
        this.mBannerListener = listener;
        this.mLovinBean = bean;
        Boolean isInitSuccess = HlAdClient.initSuccessMap.get("max");
        if (isInitSuccess == null || !isInitSuccess) {
            try {
                InitDataProcessor.initAppLovin(activity, bean.extendStr);
                HlAdClient.initSuccessMap.put("max", true);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        adBannerView = new MaxAdView(mLovinBean.adSlotId, mActivity);
        adBannerView.setExtraParameter( "allow_pause_auto_refresh_immediately", "true" );
        adBannerView.stopAutoRefresh();
        adBannerView.setListener(this);
        int width = ViewGroup.LayoutParams.MATCH_PARENT;
        int heightDp = MaxAdFormat.BANNER.getAdaptiveSize(activity).getHeight();
        int heightPx = AppLovinSdkUtils.dpToPx(activity, heightDp);
        adBannerView.setLayoutParams( new FrameLayout.LayoutParams(width, heightPx));
    }


    @Override
    public void loadAd() {
        if (adBannerView != null) {
            isShow = true;
            isClick = false;
            isAdLoad = false;
            adBannerView.loadAd();
        }
    }

    @Override
    public void showAd(ViewGroup viewGroup) {
        LogUtils.e("====bannerShowAd===");
        if (adBannerView != null && viewGroup != null) {
            viewGroup.addView(adBannerView);
            if (isShow && mBannerListener != null && mMaxAd != null) {
                isShow = false;
                mBannerListener.onDisplayAd(mLovinBean, mMaxAd.getNetworkName());
            }
        }
    }

    @Override
    public void onAdLoaded(MaxAd maxAd) {
        mMaxAd = maxAd;
        double ecpm = maxAd.getRevenue()*1000;
        mLovinBean.setEp(ecpm);
        EcpmCompareBean bean = CompareEcpmUtils.compareEcpm(mLovinBean, ecpm);
        mLovinBean.setOp(bean.getEcpm());
        if (bean.getResult()) {
            isAdLoad = true;
            mBannerListener.onAdReady(mLovinBean, InitDataProcessor.SDK_MAX, bean.getEcpm(), maxAd.getNetworkName());
        } else {
            isAdLoad = false;
            mBannerListener.onAdError("applovin: 竞价失败", 102, InitDataProcessor.SDK_MAX, mLovinBean);
        }
    }

    @Override
    public void onAdDisplayed(MaxAd maxAd) {
        if (isShow && mBannerListener != null && isAdLoad) {
            isShow = false;
            mBannerListener.onDisplayAd(mLovinBean, mMaxAd.getNetworkName());
        }
    }

    @Override
    public void onAdHidden(MaxAd maxAd) {
        if (mBannerListener != null) {
            mBannerListener.onCloseAd();
        }
    }

    @Override
    public void onAdClicked(MaxAd maxAd) {
        if (!isClick && mBannerListener != null) {
            isClick = true;
            mBannerListener.onClickAd(mLovinBean, maxAd.getNetworkName());
        }
    }

    @Override
    public void onAdLoadFailed(String s, MaxError maxError) {
        if (mBannerListener != null) {
            mBannerListener.onAdError("lovin:error===" + maxError.getMessage(), maxError.getCode(), InitDataProcessor.SDK_MAX, mLovinBean);
        }
    }

    @Override
    public void onAdDisplayFailed(MaxAd maxAd, MaxError maxError) {

    }

    @Override
    public void onAdExpanded(@NonNull MaxAd maxAd) {

    }

    @Override
    public void onAdCollapsed(@NonNull MaxAd maxAd) {

    }

    @Override
    public void release() {
        if (adBannerView != null) {
            adBannerView.destroy();
            adBannerView.stopAutoRefresh();
        }
    }
}
