package com.halo.halojuhesdk.core.channel.hlapi;

import android.app.Activity;

import com.halo.halojuhesdk.core.channel.IVideoAd;
import com.halo.halojuhesdk.core.channel.IVideoProxyListener;
import com.halo.halojuhesdk.core.common.utils.Config;
import com.halo.halojuhesdk.core.request.InitDataProcessor;
import com.halo.halojuhesdk.core.request.SlotBean;
import com.halo.halojuhesdk.sdk.HlAdClient;
import com.halomobi.sdk.HMVideoAd;
import com.halomobi.sdk.HlAdParameter;
import com.halomobi.sdk.listener.RewardVideoListener;

public class HlApiRewardVideo implements IVideoAd, RewardVideoListener {

    private SlotBean mApiBean;
    private IVideoProxyListener mListener;
    private HMVideoAd mHmRewardVideoAd;
    private boolean isShow = true;
    private boolean isClick = false;

    public HlApiRewardVideo(Activity context, SlotBean apiBean, IVideoProxyListener listener) {
        Boolean isInitSuccess = HlAdClient.initSuccessMap.get(Config.mAppId);
        if (isInitSuccess == null || !isInitSuccess) {
            try {
                InitDataProcessor.initApi(context);
                HlAdClient.initSuccessMap.put(Config.mAppId, true);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        mApiBean = apiBean;
        mListener = listener;
        mApiBean.setStartTime(System.currentTimeMillis());
        HlAdParameter adParameter = new HlAdParameter.Builder()
                .setSlotAdId(apiBean.slotId)
                .build();
        mHmRewardVideoAd = new HMVideoAd(context, adParameter, this);
        HlAdClient.containApiMap.put(mApiBean.slotId, true);
    }

    @Override
    public void loadAd() {
        isClick = false;
        isShow = true;
        if (mHmRewardVideoAd != null) {
            mHmRewardVideoAd.loadAd();
        }
    }

    @Override
    public void showAd() {
        if (mHmRewardVideoAd != null) {
            mHmRewardVideoAd.showAd();
        }
    }

    @Override
    public void onSuccess() {
        mApiBean.setEndTime(System.currentTimeMillis());
        double ecpm = mHmRewardVideoAd.getEcpm();
        mApiBean.setOp(ecpm);
        mApiBean.setEp(ecpm);
        if (ecpm < mApiBean.minPrice) {
            mListener.onAdFailed("apiVideo:价格低" + mApiBean.minPrice, 102, InitDataProcessor.SDK_API, mApiBean);
        } else {
            mListener.onSuccess(InitDataProcessor.SDK_API, mApiBean, ecpm);
        }
    }

    @Override
    public void onError(int errorCode, String errorMsg) {
        mApiBean.setEndTime(System.currentTimeMillis());
        mListener.onAdFailed("api:" + errorMsg, errorCode, InitDataProcessor.SDK_API, mApiBean);
    }

    @Override
    public void onAdShow() {
        if (isShow) {
            isShow = false;
            mListener.onAdShow(mApiBean);
        }
    }

    @Override
    public void onAdClose() {
        mListener.onAdClose();
    }

    @Override
    public void onAdSkip() {
        mListener.onSkipped();
    }

    @Override
    public void onAdClick() {
        if (!isClick) {
            isClick = true;
            mListener.onAdClick(mApiBean);
        }
    }



    @Override
    public void onPlayEnd() {
        mListener.onPlayEnd();
    }

    @Override
    public void onRewardArrived() {
        mListener.onRewardArrived();
    }

    public void onDestroy() {
        if (mHmRewardVideoAd != null) {
            mHmRewardVideoAd = null;
        }
    }

}
